import json
import re
import time

import boto3
import requests

from petsafe_smartfeed.devices import DeviceSmartFeed

URL_SF_API = "https://platform.cloud.petsafe.net/smart-feed/"
PETSAFE_CLIENT_ID = "18hpp04puqmgf5nc6o474lcp2g"
PETSAFE_REGION = "us-east-1"


class PetSafeClient:
    def __init__(
        self,
        email,
        id_token=None,
        refresh_token=None,
        access_token=None,
        session=None,
    ):
        """
        Provides a client to PetSafe API.

        Parameters
        ----------
        email : str
            Email address to authorize with PetSafe
        id_token : str, optional
            Authorization ID token provided by PetSafe
        refresh_token : str, optional
            Authorization refresh token provided by PetSafe
        access_token : str, optional
            Authorization access token provided by PetSafe
        session : str, optional
            Authorization session provided by PetSafe

        """
        self.id_token = id_token
        self.refresh_token = refresh_token
        self.access_token = access_token
        self.email = email
        self.session = session
        self.username = None
        self.token_expires_time = 0
        self.challenge_name = None
        self.client = boto3.client("cognito-idp", region_name=PETSAFE_REGION)

    @property
    def headers(self):
        """
        Client request headers with content-type JSON and authorization token.

        Returns
        -------
        dict

        """
        headers = {"Content-Type": "application/json"}

        if self.id_token is None:
            raise Exception("Not authorized! Have you requested a token?")

        if time.time() >= self.token_expires_time - 10:
            self.refresh_tokens()

        headers["Authorization"] = self.id_token

        return headers

    @property
    def feeders(self):
        """
        All feeders attached to the PetSafe account.

        Returns
        -------
        list of DeviceSmartFeed

        """
        """
        Sends a request to PetSafe's API for all feeders associated with account.

        :return: list of Feeders

        """
        response = self.api_get("feeders")
        response.raise_for_status()
        content = response.content.decode("UTF-8")
        return [
            DeviceSmartFeed(self, feeder_data) for feeder_data in json.loads(content)
        ]

    def request_code(self):
        """
        Requests an email code from PetSafe authentication.

        Returns
        -------
        dict
            Authentication response from PetSafe

        """
        response = self.client.initiate_auth(
            AuthFlow="CUSTOM_AUTH",
            ClientId=PETSAFE_CLIENT_ID,
            AuthParameters={
                "USERNAME": self.email,
                "AuthFlow": "CUSTOM_CHALLENGE",
            },
        )
        self.challenge_name = response["ChallengeName"]
        self.session = response["Session"]
        self.username = response["ChallengeParameters"]["USERNAME"]
        return response

    def request_tokens_from_code(self, code):
        """
        Requests authentication tokens from PetSafe using emailed code from
        `request_code`.

        Parameters
        ----------
        code : str
            Code provided by PetSafe via email

        Returns
        -------
        dict
            Authentication response from PetSafe

        """
        response = self.client.respond_to_auth_challenge(
            ClientId=PETSAFE_CLIENT_ID,
            ChallengeName=self.challenge_name,
            Session=self.session,
            ChallengeResponses={
                "ANSWER": re.sub(r"\D", "", code),
                "USERNAME": self.username if self.username is not None else self.email,
            },
        )
        self.id_token = response["AuthenticationResult"]["IdToken"]
        self.access_token = response["AuthenticationResult"]["AccessToken"]
        self.refresh_token = response["AuthenticationResult"]["RefreshToken"]
        self.token_expires_time = (
            time.time() + response["AuthenticationResult"]["ExpiresIn"]
        )
        return response

    def refresh_tokens(self, refresh_token=None):
        """
        Requests new authorization tokens from PetSafe using the client's or a
        provided refresh token.

        Parameters
        ----------
        refresh_token : str, optional
            Authorization refresh token provided by PetSafe

        Returns
        -------
        dict
            Authorization response from PetSafe

        """
        if refresh_token is not None:
            self.refresh_token = refresh_token

        response = self.client.initiate_auth(
            AuthFlow="REFRESH_TOKEN_AUTH",
            AuthParameters={"REFRESH_TOKEN": self.refresh_token},
            ClientId=PETSAFE_CLIENT_ID,
        )

        if "Session" in response:
            self.session = response["Session"]

        self.id_token = response["AuthenticationResult"]["IdToken"]
        self.access_token = response["AuthenticationResult"]["AccessToken"]
        self.refresh_token = response["AuthenticationResult"]["RefreshToken"]
        self.token_expires_time = (
            time.time() + response["AuthenticationResult"]["ExpiresIn"]
        )
        return response

    def api_post(self, path="", data=None):
        """
        Sends a POST request to PetSafe.

        Parameters
        ----------
        path : str
            URL path on the API (it is prepended by the API URL)
        data
            JSON data to send on the request

        Returns
        -------
        Response
            Response received from PetSafe

        Examples
        --------
        >>> client = PetSafeClient("example@email.com", refresh_token="XXXX")
        >>> feeder = client.feeders[0]
        >>> response = client.api_post(path=feeder.api_path + "meals", data={
        ...     "amount": 1,  # feed 1/8th
        ...     "slow_feed": False
        ... })

        """
        return requests.post(URL_SF_API + path, headers=self.headers, json=data)

    def api_get(self, path=""):
        """
        Sends a GET request to PetSafe.

        Parameters
        ----------
        path : str
            URL path on the API (it is prepended by the API URL)

        Returns
        -------
        Response
            Response received from PetSafe

        Examples
        --------
        >>> client = PetSafeClient("example@email.com", refresh_token="XXXX")
        >>> feeders_raw = client.api_get(path="feeders")

        """
        return requests.get(URL_SF_API + path, headers=self.headers)

    def api_put(self, path="", data=None):
        """
        Sends a PUT request to PetSafe.

        Parameters
        ----------
        path : str
            URL path on the API (it is prepended by the API URL)
        data
            JSON data to send on the request

        Returns
        -------
        Response
            Response received from PetSafe

        Examples
        --------
        >>> client = PetSafeClient("example@email.com", refresh_token="XXXX")
        >>> feeder = client.feeders[0]
        >>> response = client.api_put(
        ...    feeder.api_path + "schedules/1",
        ...    data={
        ...        "time": "16:35",  # time in 24 hour format
        ...        "amount": 1,
        ...    },
        ...)

        """
        return requests.put(URL_SF_API + path, headers=self.headers, json=data)

    def api_delete(self, path=""):
        """
        Sends a DELETE request to PetSafe.

        Parameters
        ----------
        path : str
            URL path on the API (it is prepended by the API URL)

        Returns
        -------
        Response
            Response received from PetSafe

        Examples
        --------
        >>> client = PetSafeClient("example@email.com", refresh_token="XXXX")
        >>> feeder = client.feeders[0]
        >>> response = client.api_delete(feeder.api_path + "schedules/1")

        """
        return requests.delete(URL_SF_API + path, headers=self.headers)



import argparse
import sys

import petsafe_smartfeed as sf

# create parser for arguments
parser = argparse.ArgumentParser(usage="python -m petsafe_smartfeed email [-t email_code]")
parser.add_argument("email", help="account email address")

# if no arguments specified, show help
if len(sys.argv) < 2:
    parser.print_help()
    sys.exit(1)

# parse for arguments
args = parser.parse_args()

client = sf.PetSafeClient(email=args.email)
client.request_code()
print("Code requested, please check your email.")
print("")

code = input("Enter email code: ")
client.request_tokens_from_code(code)

print("")
print("IdToken:")
print(client.id_token)
print("")
print("AccessToken:")
print(client.access_token)
print("")
print("RefreshToken:")
print(client.refresh_token)



import json
from warnings import warn


def get_feeders(client):
    """
    Sends a request to PetSafe for all feeders associated with an account.

    .. deprecated:: 2.3
              `get_feeders` will be removed in the next version.
              Use `PetSafeClient.feeders` instead.

    Parameters
    ----------
    client : PetSafeClient
        Authorized PetSafe client

    Returns
    -------
    list of DeviceSmartFeed

    """
    warn(
        "`get_feeders` will be removed in the next version. Use `PetSafeClient.feeders` instead.",
        DeprecationWarning,
        stacklevel=2,
    )
    response = client.api_get("feeders")
    response.raise_for_status()
    content = response.content.decode("UTF-8")
    return [DeviceSmartFeed(client, feeder_data) for feeder_data in json.loads(content)]


class DeviceSmartFeed:
    def __init__(self, client, data):
        """
        PetSafe SmartFeed device.

        Parameters
        ----------
        client : PetSafeClient
            Authorized PetSafe client
        data : dict
            PetSafe's provided JSON feeder data

        Notes
        -----
        It is recommended you retrieve this using `PetSafeClient.feeders`.

        """
        self.client = client
        self.data = data

    def __str__(self):
        """
        An alias of `to_json`.

        """
        return self.json

    @property
    def json(self):
        """
        Feeder data formatted as JSON.

        """
        return json.dumps(self.data, indent=2)

    def update_data(self):
        """
        Updates `self.data` to the feeder's current state from PetSafe.

        """
        response = self.client.api_get(self.api_path)
        response.raise_for_status()
        self.data = json.loads(response.content.decode("UTF-8"))

    def put_setting(self, setting, value, force_update=False):
        """
        Changes a value of the feeder's settings.

        Parameters
        ----------
        setting : str
            Name of setting to be changed
        value
            Value of setting to apply
        force_update : bool, optional
            If True, updates ALL device data after PUT.
            Defaults to False.

        """
        response = self.client.api_put(
            self.api_path + "settings/" + setting,
            data={
                "value": value,
            },
        )
        response.raise_for_status()

        if force_update:
            self.update_data()
        else:
            self.data["settings"][setting] = value

    def get_messages_since(self, days=7):
        """
        Requests feeder messages since a specified date.

        Parameters
        ----------
        days : int, optional
            Number of days to request back.
            Default to 7.

        Returns
        -------
        dict
            JSON data returned from PetSafe

        """
        response = self.client.api_get(self.api_path + "messages?days=" + str(days))
        response.raise_for_status()
        return json.loads(response.content.decode("UTF-8"))

    def get_last_feeding(self):
        """
        Requests the most recent feeding message within past 7 days.

        Returns
        -------
        dict or None
            JSON data returned from PetSafe

        """
        messages = self.get_messages_since()
        for message in messages:
            if message["message_type"] == "FEED_DONE":
                return message
        return None

    def feed(self, amount=1, slow_feed=None, update_data=True):
        """
        Requests the feeder to start a feeding.

        Parameters
        ----------
        amount : int
            Amount to feed in increments of 1/8
        slow_feed : bool, optional
            If True, will use slow feeding.
            Defaults to current setting.
        update_data : bool
            If True, updates ALL device data after the request.
            Defaults to True.

        """
        if slow_feed is None:
            slow_feed = self.data["settings"]["slow_feed"]

        response = self.client.api_post(
            self.api_path + "meals",
            data={
                "amount": amount,
                "slow_feed": slow_feed,
            },
        )
        response.raise_for_status()

        if update_data:
            self.update_data()

    def repeat_feed(self):
        """
        Repeats the last feeding.

        """
        last_feeding = self.get_last_feeding()
        self.feed(last_feeding["amount"])

    def prime(self):
        """
        Feeds 5/8 cups to prime the feeder.

        """
        self.feed(5, False)

    def get_schedules(self):
        """
        Requests all scheduled feeds.

        Returns
        -------
        dict
            JSON data returned from PetSafe

        """
        response = self.client.api_get(self.api_path + "schedules")
        response.raise_for_status()
        return json.loads(response.content.decode("UTF-8"))

    def schedule_feed(self, time="00:00", amount=1, update_data=True):
        """
        Adds scheduled feed with time and food amount.

        .. deprecated:: 2.3
                  `schedule_feed` will be removed in the next version.
                  Use `add_schedule` instead.

        Parameters
        ----------
        time : str
            Time to dispense the food in 24 hour notation with colon separation (e.g. 16:35 for 4:35PM)
        amount : int
            Amount to feed in increments of 1/8
        update_data
            If True, updates ALL device data after the request.
            Defaults to True.

        Returns
        -------
        dict
            JSON data returned from PetSafe
            (Unique ID of the scheduled feed)

        """
        warn(
            "`schedule_feed` will be remvoved in the next version. Use `add_schedule` instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.add_schedule(time, amount, update_data)

    def add_schedule(self, time="00:00", amount=1, update_data=True):
        """
        Adds scheduled feed with time and food amount.

        Parameters
        ----------
        time : str
            Time to dispense the food in 24 hour notation with colon separation (e.g. 16:35 for 4:35PM)
        amount : int
            Amount to feed in increments of 1/8
        update_data
            If True, updates ALL device data after the request.
            Defaults to True.

        Returns
        -------
        dict
            JSON data returned from PetSafe
            (Unique ID of the scheduled feed)

        """
        response = self.client.api_post(
            self.api_path + "schedules",
            data={
                "time": time,
                "amount": amount,
            },
        )
        response.raise_for_status()

        if update_data:
            self.update_data()

        return json.loads(response.content.decode("UTF-8"))

    def modify_schedule(self, time="00:00", amount=1, schedule_id="", update_data=True):
        """
        Modifies the food amount and time of the specified scheduled feed ID.

        Parameters
        ----------
        time : str
            Time to dispense the food in 24 hour notation with colon separation (e.g. 16:35 for 4:35PM)
        amount : int
            Amount to feed in increments of 1/8
        schedule_id : str
            Unique ID of the schedule to modify
        update_data
            If True, updates ALL device data after the request.
            Defaults to True.

        """
        response = self.client.api_put(
            self.api_path + "schedules/" + schedule_id,
            data={
                "time": time,
                "amount": amount,
            },
        )
        response.raise_for_status()

        if update_data:
            self.update_data()

    def delete_schedule(self, schedule_id="", update_data=True):
        """
        Deletes the specified scheduled feed ID.

        Parameters
        ----------
        schedule_id : str
            Unique ID of the schedule to modify
        update_data
            If True, updates ALL device data after the request.
            Defaults to True.

        """
        response = self.client.api_delete(self.api_path + "schedules/" + schedule_id)
        response.raise_for_status()

        if update_data:
            self.update_data()

    def delete_all_schedules(self, update_data=True):
        """
        Deletes all scheduled feeds.

        Parameters
        ----------
        update_data
            If True, updates ALL device data after the request.
            Defaults to True.

        """
        response = self.client.api_delete(self.api_path + "schedules")
        response.raise_for_status()

        if update_data:
            self.update_data()

    @property
    def api_name(self):
        """
        Feeder's thing_name from the API.

        """
        return self.data["thing_name"]

    @property
    def api_path(self):
        """
        Feeder's path on the API.

        """
        return "feeders/" + self.api_name + "/"

    @property
    def id(self):
        """
        Feeder's ID.

        """
        return self.data["id"]

    @property
    def battery_voltage(self):
        """
        Feeder's calculated current battery voltage.

        """
        try:
            return round(int(self.data["battery_voltage"]) / 32767 * 7.2, 3)
        except ValueError:
            return -1

    @property
    def battery_level(self):
        """
        Feeder's current battery level on a scale of 0-100.
        Will return 0 if no batteries installed.

        """
        if not self.data["is_batteries_installed"]:
            return 0
        minVoltage = 22755
        maxVoltage = 29100
        return round(
            max(
                (100 * (int(self.data["battery_voltage"]) - minVoltage))
                / (maxVoltage - minVoltage),
                0,
            )
        )

    @property
    def paused(self):
        """
        If True, the feeder will not follow its scheduling.

        Setting this property automatically updates via the API.
        This does not update ALL device data.

        """
        return self.data["settings"]["paused"]

    @paused.setter
    def paused(self, value):
        self.put_setting("paused", value)

    @property
    def slow_feed(self):
        """
        If true, the feeder will dispense food slowly.

        Setting this property changes the setting via the API.
        This does not update ALL device data.

        """
        return self.data["settings"]["slow_feed"]

    @slow_feed.setter
    def slow_feed(self, value):
        self.put_setting("slow_feed", value)

    @property
    def child_lock(self):
        """
        If true, the feeder's physical button is disabled.

        Setting this property changes the setting via the API.
        This does not update ALL device data.

        """
        return self.data["settings"]["child_lock"]

    @child_lock.setter
    def child_lock(self, value):
        self.put_setting("child_lock", value)

    @property
    def friendly_name(self):
        """
        The feeder's display name.

        Setting this property changes the setting via the API.
        This does not update ALL device data.

        """
        return self.data["settings"]["friendly_name"]

    @friendly_name.setter
    def friendly_name(self, value):
        self.put_setting("friendly_name", value)

    @property
    def pet_type(self):
        """
        Feeder's pet type.

        Setting this property changes the setting via the API.
        This does not update ALL device data.

        """
        return self.data["settings"]["pet_type"]

    @pet_type.setter
    def pet_type(self, value):
        self.put_setting("pet_type", value)

    @property
    def food_sensor_current(self):
        """
        Feeder's food sensor status.

        """
        return self.data["food_sensor_current"]

    @property
    def food_low_status(self):
        """
        Feeder's food low status.
        (0 if Full, 1 if Low, 2 if Empty)

        """
        return int(self.data["is_food_low"])



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="petsafe_smartfeed",
    version="2.3",
    author="Jordan Stremming",
    license="MIT",
    author_email="jcstremming@gmail.com",
    description="Provides ability to connect and control a PetSafe Smart Feed device using the PetSafe-SmartFeed API.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/techzune/petsafe_smartfeed",
    packages=setuptools.find_packages(),
    install_requires=["requests", "boto3"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)
